@@ -1,4 +1,5 @@
 from OpenGL import GL
+from PIL import Image
 
 from ..shaders import Shader
 from .view import View
@@ -57,9 +58,11 @@ class View120(View):
         self.shader_grid.uniform4x4("transform", transform)
         self.shader_grid.release()
 
-    def resize(self, w, h):
-        projection = self.camera.projection(w, h)
+    def update_projection(self, w=None, h=None):
+        w = w or self.app.width
+        h = h or self.app.height
 
+        projection = self.camera.projection(w, h)
         self.shader_model.bind()
         self.shader_model.uniform4x4("projection", projection)
         self.shader_model.release()
@@ -76,6 +79,9 @@ class View120(View):
         self.shader_grid.uniform4x4("projection", projection)
         self.shader_grid.release()
 
+    def resize(self, w, h):
+        self.update_projection(w, h)
+
     def sort_objects_from_viewworld(self, viewworld):
         """Sort objects by the distances from their bounding box centers to camera location"""
         opaque_objects = []
@@ -96,15 +102,14 @@ class View120(View):
         return opaque_objects + list(transparent_objects)
 
     def paint(self):
-        projection = self.camera.projection(self.app.width, self.app.height)
         viewworld = self.camera.viewworld()
+        if self.current != self.PERSPECTIVE:
+            self.update_projection()
 
         # Draw instance maps
         if self.app.selector.enabled:
             self.shader_instance.bind()
             # set projection matrix
-            if self.current != self.PERSPECTIVE:
-                self.shader_instance.uniform4x4("projection", projection)
             self.shader_instance.uniform4x4("viewworld", viewworld)
             if self.app.selector.select_from == "pixel":
                 self.app.selector.instance_map = self.paint_instances()
@@ -116,8 +121,6 @@ class View120(View):
 
         # Draw grid
         self.shader_grid.bind()
-        if self.current != self.PERSPECTIVE:
-            self.shader_grid.uniform4x4("projection", projection)
         self.shader_grid.uniform4x4("viewworld", viewworld)
         if self.app.selector.wait_for_selection_on_plane:
             self.app.selector.uv_plane_map = self.paint_plane()
@@ -128,28 +131,36 @@ class View120(View):
 
         # Draw model objects in the scene
         self.shader_model.bind()
-        if self.current != self.PERSPECTIVE:
-            self.shader_model.uniform4x4("projection", projection)
         self.shader_model.uniform4x4("viewworld", viewworld)
         for obj in self.sort_objects_from_viewworld(viewworld):
-            obj.draw(self.shader_model, self.mode == "wireframe", self.mode == "lighted")
+            if obj.visible:
+                obj.draw(self.shader_model, self.mode == "wireframe", self.mode == "lighted")
         self.shader_model.release()
 
         # draw text sprites
         self.shader_text.bind()
-        if self.current != self.PERSPECTIVE:
-            self.shader_text.uniform4x4("projection", projection)
         self.shader_text.uniform4x4("viewworld", viewworld)
         for guid in self.objects:
             obj = self.objects[guid]
             if isinstance(obj, TextObject):
-                obj.draw(self.shader_text)
+                if obj.visible:
+                    obj.draw(self.shader_text)
         self.shader_text.release()
 
         # draw 2D box for multi-selection
         if self.app.selector.select_from == "box":
             self.shader_model.draw_2d_box(self.app.selector.box_select_coords, self.app.width, self.app.height)
 
+        if self.app.record:
+            r = self.devicePixelRatio()
+            x, y, width, height = 0, 0, self.app.width, self.app.height
+            buffer = GL.glReadPixels(x*r, y*r, width*r, height*r, GL.GL_RGB, GL.GL_UNSIGNED_BYTE)
+            arr = np.frombuffer(buffer, dtype=np.uint8).reshape(height*r, width*r, 3)
+            arr = arr[::-r, ::r, :]
+            im = Image.fromarray(arr)
+            print("recording frame:", self.app.frame_count)
+            self.app.recorded_frames.append(im)
+
     def paint_instances(self, cropped_box=None):
         GL.glDisable(GL.GL_POINT_SMOOTH)
         GL.glDisable(GL.GL_LINE_SMOOTH)
@@ -162,7 +173,8 @@ class View120(View):
         for guid in self.objects:
             obj = self.objects[guid]
             if hasattr(obj, "draw_instance"):
-                obj.draw_instance(self.shader_instance, self.mode == "wireframe")
+                if obj.visible:
+                    obj.draw_instance(self.shader_instance, self.mode == "wireframe")
         # create map
         r = self.devicePixelRatio()
         instance_buffer = GL.glReadPixels(x*r, y*r, width*r, height*r, GL.GL_RGB, GL.GL_UNSIGNED_BYTE)
